@@ -101,22 +101,13 @@ namespace Rcpp {
     }
 
     inline SEXP Rcpp_ReplaceObject(SEXP x, SEXP y) {
-        if (Rf_isNull(x)) {
+
+        // if we are setting to the same SEXP as we already have, do nothing
+        if (x != y) {
+            Rcpp_ReleaseObject(x);
             Rcpp_PreserveObject(y);
-        } else if (Rf_isNull(y)) {
-            Rcpp_ReleaseObject(x); 	// #nocov
-        } else {
-            // if we are setting to the same SEXP as we already have, do nothing
-            if (x != y) {
-
-                // the previous SEXP was not NULL, so release it
-                Rcpp_ReleaseObject(x);
-
-                // the new SEXP is not NULL, so preserve it
-                Rcpp_PreserveObject(y);
-
-            }
         }
+
         return y;
     }
 
